package be.iminds.ilabt.jfed.highlevel.jobs.report;

import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.jobs.AbstractJob;
import be.iminds.ilabt.jfed.highlevel.util.LogEntryListener;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import javax.annotation.Nonnull;
import java.time.Instant;

/**
 * User: twalcari
 * Date: 12/9/2015
 * Time: 16:02
 */
public class JobReport implements LogEntryListener {
    private final AbstractJob<?> job;

    private final ObjectProperty<Instant> startTime = new SimpleObjectProperty<>();
    private final ObjectProperty<Instant> endTime = new SimpleObjectProperty<>();

    private final ObservableList<JobStateReport> jobStateReports = FXCollections.observableArrayList();
    private JobStateReport currentStateReport = null;

    private final ObservableList<LogOutput.LogEntry> statelessLogLines =
            FXCollections.synchronizedObservableList(FXCollections.observableArrayList());


    public JobReport(@Nonnull AbstractJob<?> job) {
        this.job = job;

        assert job.getState() == null : "Cannot attach report to active job";

        job.stateProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue != null) {
                final JobStateReport jobStateReport = new JobStateReport(newValue);
                newValue.registerStateFinishedListener(state -> jobStateReport.registerEnd(state.getStatus()));
                jobStateReports.add(jobStateReport);
                currentStateReport = jobStateReport;
            } else {
                currentStateReport = null;
            }
        });

        job.addTaskExecutionFinishedListener(this::onTaskExecutionFinished);
    }

    public void registerStart() {
        this.startTime.set(Instant.now());
    }

    public void registerEnd() {
        this.endTime.set(Instant.now());

    }

    private void onTaskExecutionFinished(TaskExecution<?> taskExecution) {
        assert currentStateReport != null;
        currentStateReport.addTaskExecution(taskExecution);
    }

    public ObservableList<JobStateReport> getJobStateReports() {
        return jobStateReports;
    }

    public void log(@Nonnull LogOutput.LogEntry logEntry) {
        if (currentStateReport != null)
            currentStateReport.addLogLine(logEntry);
        else
            statelessLogLines.add(logEntry);
    }

    public ObservableList<LogOutput.LogEntry> getStatelessLogLines() {
        return statelessLogLines;
    }

    public String getName(){
        return job.getName();
    }

    public Instant getStartTime() {
        return startTime.get();
    }

    public Instant getEndTime() {
        return endTime.get();
    }
}
